#ifndef STARSIRDBSegment_h
#define STARSIRDBSegment_h


#include "interfaces/STARSTypes.h"
#include "elfio/elfio.hpp"


/* 
 * There is some terminology mismatch between IDA's use of 
 * "segment" and ELF's use of "segment".
 * This file will inherently be confusing as both uses are in play. 
 */
class STARS_IRDB_Segment_t : public STARS_Segment_t
{
	public:
		// create
		STARS_IRDB_Segment_t(ELFIO::section* seg, int p_segno) : the_seg(seg), segno(p_segno) {}

		// get starting address
        	virtual STARS_ea_t get_startEA()
		{ return the_seg->get_address(); }

		// get ending address
        	virtual STARS_ea_t get_endEA()
		{ return the_seg->get_address()+the_seg->get_size(); }

		// return if the EXE bit is set.
        	virtual bool IsCodeSegment()  
		{ return (the_seg->get_flags() & SHF_EXECINSTR) == SHF_EXECINSTR; }

        	virtual bool IsDataSegment() 
		{ return IsReadableSegment() && IsWriteableSegment(); }

        	virtual bool IsBSSSegment() 
		{ return (the_seg->get_type() & SHT_NOBITS) == SHT_NOBITS; }

        	virtual bool IsReadableSegment() 
		{ return (the_seg->get_flags() & SHF_ALLOC) == SHF_ALLOC; }

        	virtual bool IsWriteableSegment()
		{ return (the_seg->get_flags() & SHF_WRITE) == SHF_WRITE; }

		virtual STARS_ssize_t GetSegmentName(char* name, int len) 
		{
			strncpy(name,the_seg->get_name().c_str(), len);
			return the_seg->get_name().length();
		}


		virtual int GetSegmentNumber() { return segno; }

		// these aren't used, impl. later if we know what it means. 
        	virtual bool IsXternSegment() { assert(0); /* leave as error */}
        	virtual bool IsCommonSegment() { assert(0); /* leave as error */}

	private:
		ELFIO::section* the_seg;
		int segno;
		
};

#endif
